<p class="h4" title="{{ var_name }}"><a
        href="#pp_var_{{ anchor_id }}">{{ var_name }}</a><br/>
    <small>{{ var_type }}</small>
</p>
{% for alert in alerts %}
    <code>{{ alert }}</code><br />
{% endfor %}
<p class="variable-description">
    {{ description }}
</p>