<div class="col-sm-{{ nb_col }} collapse in" id="minifreqtable{{ varid }}">
    <table class="mini freq">
        {% for row in rows %}
            <tr class="{{ row['extra_class'] }}">
                <th width="50%">
                      {% if not redact %}
                        {{ row['label'] | fmt }}
                      {% endif %}
                </th>
                <td width="50%">
                    <div class="bar"
                         style="width:{{ row['width'] | fmt_percent(edge_cases=False) }}"
                         data-toggle="tooltip"
                         data-placement="right"
                         data-html="true"
                         data-delay=500 >
                        {% if row['width'] > 0.10 %}
                            {{ row['count'] }}&nbsp;
                        {% else %}
                            &nbsp;
                        {% endif %}
                    </div>
                    {% if row['width'] <= 0.10 %}
                        {{ row['count'] }}
                    {% endif %}
                </td>
            </tr>
        {% endfor %}
    </table>
</div>