<table class="table table-condensed list-warnings">
    <p class="h4">Alerts</p>
    {% for alert in alerts %}
        {% if alert.alert_type.name != "REJECTED" %}
            <tr{% if loop.first %} style="border-top:0"{% endif %}>
                <td>
                    {% include 'alerts/alert_' + alert.alert_type.name | lower  + '.html'  %}
                </td>
                <td>
                    <span class="label label-{{ styles[alert.alert_type.name | lower] }}">{{ alert.alert_type.name | lower | capitalize | replace("_", " ") }}</span>
                </td>
            </tr>
        {% endif %}
    {% endfor %}
</table>